import { Inject, Injectable, Scope } from '@nestjs/common';
import { INQUIRER } from '@nestjs/core';

import { LOG_LEVEL, Logger, LoggerBaseKey } from './logger.types';

@Injectable({ scope: Scope.TRANSIENT })
export class LoggerService {
	private sourceClass: string;

	private context: string;

	constructor(
		@Inject(LoggerBaseKey) private logger: Logger,
		@Inject(INQUIRER) parentClass: object
	) {
		this.sourceClass = parentClass?.constructor.name;
	}

	public log(level: LOG_LEVEL, message: string | Error, context: string, data?: unknown) {
		return this.logger.log(level, message, context, this.getLogData(data));
	}

	public debug(message: string, context: string, data?: unknown) {
		return this.logger.debug(message, context, this.getLogData(data));
	}

	public info(message: string, context: string, data?: unknown) {
		return this.logger.info(message, context, this.getLogData(data));
	}

	public warn(message: string | Error, context: string, data?: unknown) {
		return this.logger.warn(message, context, this.getLogData(data));
	}

	public error(message: string | Error, context: string, data?: unknown) {
		return this.logger.error(message, context, this.getLogData(data));
	}

	public fatal(message: string | Error, context: string, data?: unknown) {
		return this.logger.fatal(message, context, this.getLogData(data));
	}

	public emergency(message: string | Error, context: string, data?: unknown) {
		return this.logger.emergency(message, context, this.getLogData(data));
	}

	private getLogData(data?: unknown): unknown {
		return data;
	}
}
